@@ -462,11 +462,7 @@ int
 fts_write_object_id(
 /*================*/
 	ib_id_t		id,		/*!< in: a table/index id */
-	char*		str,		/*!< in: buffer to write the id to */
-	bool		hex_format MY_ATTRIBUTE((unused)))
-					/*!< in: true for fixed hex format,
-					false for old ambiguous format */
-	MY_ATTRIBUTE((nonnull));
+	char*		str);		/*!< in: buffer to write the id to */
 /******************************************************************//**
 Read the table id from the string generated by fts_write_object_id().
 @return TRUE if parse successful */
@@ -516,6 +512,6 @@ fts_config_create_index_param_name(
 	const dict_index_t*	index)	/*!< in: index for config */
 	MY_ATTRIBUTE((nonnull, malloc, warn_unused_result));
 
-#include "fts0priv.ic"
+#include "fts0priv.inl"
 
 #endif /* INNOBASE_FTS0PRIV_H */
